// ignore_for_file: public_member_api_docs

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:healthrian_visualization_support/'
    'healthrian_visualization_support.dart';

part 'control_panel.g.dart';

_$ControlPanelCWProxy get ControlPanelWearEcg =>
    ControlPanel(module: BLEManager().wearEcg12.module).copyWith;

_$ControlPanelCWProxy get ControlPanelCardioClick =>
    ControlPanel(module: BLEManager().cardioClick.module).copyWith;

_$ControlPanelCWProxy get ControlPanelWearT =>
    ControlPanel(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class ControlPanel extends StatelessWidget {
  const ControlPanel({
    required this.module,
    this.turnOn,
    this.turnOff,
    this.recordStart,
    this.recordPause,
    this.recordStop,
    super.key,
  });

  final ModuleInterface module;

  final void Function()? turnOn;
  final void Function()? turnOff;

  final void Function()? recordStart;
  final void Function()? recordPause;
  final void Function()? recordStop;

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.end,
      crossAxisAlignment: CrossAxisAlignment.end,
      children: [
        FloatingActionButton(
          onPressed: () async {
            await module.ble.connectAndRegister();
            turnOn?.call();
          },
          child: const Icon(Icons.bluetooth),
        ),
        const SizedBox(height: 8),
        FloatingActionButton(
          onPressed: () async {
            await module.ecg?.start();
            await module.oxi?.start();
            recordStart?.call();
          },
          child: const Icon(Icons.play_arrow),
        ),
        const SizedBox(height: 8),
        FloatingActionButton(
          onPressed: () async {
            await module.ecg?.pause();
            await module.oxi?.pause();
            recordPause?.call();
          },
          child: const Icon(Icons.pause),
        ),
        const SizedBox(height: 8),
        FloatingActionButton(
          onPressed: () async {
            await module.ecg?.stop();
            await module.oxi?.stop();
            module.ecg?.initStream();
            module.oxi?.initStream();

            recordStop?.call();
          },
          child: const Icon(Icons.stop),
        ),
        const SizedBox(height: 8),
        FloatingActionButton(
          onPressed: () {
            module.ble.disconnectDevice();
            turnOff?.call();
          },
          child: const Icon(Icons.bluetooth_disabled),
        ),
      ],
    );
  }
}
