// ignore_for_file: public_member_api_docs, lines_longer_than_80_chars

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_animate/flutter_animate.dart';
import 'package:healthrian_visualization_support/'
    'healthrian_visualization_support.dart';
import 'package:rxdart/rxdart.dart';

typedef MergeScanConnect = (
  Map<DeviceIdentifier, ScanResult>,
  List<BluetoothDevice>
);

class BLEList extends StatefulWidget {
  const BLEList({super.key});

  @override
  State<BLEList> createState() => _BLEListState();
}

class _BLEListState extends State<BLEList> {
  final _controller = StreamController<Map<DeviceIdentifier, ScanResult>>();
  late final _streamScan = _controller.stream.asBroadcastStream();
  final _scanMap = <DeviceIdentifier, ScanResult>{};

  late final Stream<List<BluetoothDevice>> _streamConnect;
  late final Stream<MergeScanConnect> _streamMerge;

  @override
  void initState() {
    super.initState();
    _streamConnect = Rx.merge([
      Stream.value(FlutterBluePlus.connectedDevices),
      Stream.periodic(500.ms, (_) => FlutterBluePlus.connectedDevices),
    ]).distinct();

    _streamMerge = _streamScan
        .zipWith(_streamConnect, (t, s) => (t, s))
        .asBroadcastStream();

    _rescan();
  }

  @override
  void dispose() {
    super.dispose();
    _controller.close();
  }

  Future<void> _rescan() async {
    if (FlutterBluePlus.isScanningNow) return;
    // if (isClear) _scanMap.clear();
    final subscription = FlutterBluePlus.scanResults.listen(
      (results) {
        for (ScanResult r in results) {
          if (_scanMap[r.device.remoteId]?.device.platformName.isEmpty == false)
            continue;
          _scanMap[r.device.remoteId] = r;
          _controller.add(_scanMap);
        }
      },
    );

    final timeout = const Duration(seconds: 4);
    await FlutterBluePlus.startScan();
    await Future.delayed(timeout);
    await FlutterBluePlus.stopScan();
    subscription.cancel();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: StreamBuilder(
        stream: _streamMerge,
        builder: (context, snapshot) {
          final scanned = [...?snapshot.data?.$1.values]
            ..sort((a, b) => b.rssi - a.rssi);
          final connected = [...?snapshot.data?.$2];
          return ListView.separated(
            itemCount: connected.length + scanned.length,
            separatorBuilder: (_, __) => const Divider(),
            itemBuilder: (context, index) {
              /// connected
              if (index < connected.length) {
                return ListTile(
                  onTap: () =>
                      connected[index].disconnect().whenComplete(_rescan),
                  title: Text(connected[index].platformName),
                  subtitle: Text(connected[index].remoteId.str),
                );
              }

              /// scanned
              final mIndex = index - connected.length;
              return ListTile(
                onTap: () => scanned[mIndex].device.connect().whenComplete(
                  () {
                    _scanMap.remove(scanned[mIndex].device.remoteId);
                    _controller.add(_scanMap);
                  },
                ),
                title: Text(scanned[mIndex].device.platformName),
                subtitle: Text(scanned[mIndex].device.remoteId.str),
                leading: Text(scanned[mIndex].rssi.toString()),
                trailing: Text(scanned[mIndex].timeStamp.toString()),
              );
            },
          );
        },
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          _scanMap.clear();
          _rescan();
        },
      ),
    );
  }
}
